@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Task.py 295 2010-07-21 19:55:15Z st-cea $
+# $Id: Task.py 328 2010-08-31 23:08:35Z st-cea $
 
 """
 ClusterShell Task module.
@@ -548,18 +548,49 @@ class Task(object):
         self._add_port(port)
         return port
 
-    @tasksyncmethod()
     def timer(self, fire, handler, interval=-1.0, autoclose=False):
         """
-        Create task's timer.
+        Create a timer bound to this task that fires at a preset time
+        in the future by invoking the ev_timer() method of `handler'
+        (provided EventHandler object). Timers can fire either only
+        once or repeatedly at fixed time intervals. Repeating timers
+        can also have their next firing time manually adjusted.
+
+        The mandatory parameter `fire' sets the firing delay in seconds.
+        
+        The optional parameter `interval' sets the firing interval of
+        the timer. If not specified, the timer fires once and then is
+        automatically invalidated.
+
+        Time values are expressed in second using floating point
+        values. Precision is implementation (and system) dependent.
+
+        The optional parameter `autoclose', if set to True, creates
+        an "autoclosing" timer: it will be automatically invalidated
+        as soon as all other non-autoclosing task's objects (workers,
+        ports, timers) have finished. Default value is False, which
+        means the timer will retain task's runloop until it is
+        invalidated.
+
+        Return a new EngineTimer instance.
+
+        See ClusterShell.Engine.Engine.EngineTimer for more details.
         """
         assert fire >= 0.0, \
             "timer's relative fire time must be a positive floating number"
         
         timer = EngineTimer(fire, interval, autoclose, handler)
-        self._engine.add_timer(timer)
+        # The following method may be sent through msg port (async
+        # call) if called from another task.
+        self._add_timer(timer)
+        # always return new timer (sync)
         return timer
 
+    @tasksyncmethod()
+    def _add_timer(self, timer):
+        """Add a timer to task engine (thread-safe)."""
+        self._engine.add_timer(timer)
+
     @tasksyncmethod()
     def schedule(self, worker):
         """
